@@ -30,6 +30,7 @@ import urllib
 import https
 import time
 import codecs
+import wsgiref.handlers
 
 import config
 import utils
@@ -37,7 +38,7 @@ import remote
 import local
 
 
-VERSION = '0.4.7'
+VERSION = '0.4.8'
 
 
 def hook_setup(cmd):
@@ -169,11 +170,21 @@ def run_command(cmd, internal=False, retval=False, progress=False):
         'X-DotCloud-Version': VERSION
         })
     trace_id = None
+    req.localtimestamp = wsgiref.handlers.format_date_time(time.time())
+    req.remotetimestamp = None
     try:
+        # Before doing the actual request (which might blow up in many
+        # different ways), let's connect, find out the remote address
+        # and port, and memorize them for further reference.
+        if not hasattr(req, 'endpoint'):
+            req.endpoint = None
+            req.connect()
+            req.endpoint = req.sock.getpeername()
         req.request('GET', query, headers=headers)
         resp = req.getresponse()
         info = resp.getheader('X-Dotcloud-Info')
         trace_id = resp.getheader('X-Dotcloud-TraceID')
+        req.remotetimestamp = resp.getheader('Date')
         data = resp.read()
         req.close()
         if _export is True:
@@ -194,14 +205,17 @@ def run_command(cmd, internal=False, retval=False, progress=False):
         elif progress:
             sys.stderr.write('.')
     except socket.error, e:
-        utils.die('Cannot reach DotCloud service ("{0}").\n' \
-                'Please check the connectivity and try again.'.format(str(e)))
+        utils.die('Cannot reach DotCloud service ("{0}").\n'
+                  '{1}\n{2}\n'
+                  'Please check the connectivity and try again.'
+                  .format(e, get_req_endpoint(req), get_req_timestamp(req)))
     except Exception, e:
-        message = 'DotCloud service unavailable ("{0}").\n' \
-                'Data received:\n---\n{1}\n---\n' \
-                'Please try again later. ' \
-                'If the problem persists, send an email to support@dotcloud.com'.format(
-                    e, data)
+        message = ('DotCloud service unavailable ("{0}").\n'
+                   '{1}\n{2}\n'
+                   'Data received:\n---\n{3}\n---\n'
+                   'Please try again later. '
+                   'If the problem persists, send an email to support@dotcloud.com'
+                   .format(e, get_req_endpoint(req), get_req_timestamp(req), data))
         if trace_id:
             message += '\nand include the following TraceID: {0}'.format(trace_id)
         else:
@@ -221,6 +235,19 @@ def run_command(cmd, internal=False, retval=False, progress=False):
         sys.exit(1)
     sys.exit(int(data['type'] != 'success'))
 
+def get_req_timestamp(req):
+    if req.remotetimestamp:
+        return 'The remote timestamp is {0}.'.format(req.remotetimestamp)
+    else:
+        return 'The local timestamp is {0}.'.format(req.localtimestamp)
+
+def get_req_endpoint(req):
+    if req.endpoint is None:
+        return 'I could not establish a healthy connection to the API.'
+    host, port = req.endpoint
+    return ('The API endpoint used for this request was {0}:{1}.'
+            .format(host, port))
+
 def unicode_output_patch():
     for stream_name in ('stdout', 'stderr'):
         stream = getattr(sys, stream_name)
